<?php
namespace App\Controller;

use App\Exception\SqlException as SqlException;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

class WebhookController {  

    const MERCHANT_API_BASEURL = "https://hm.merchant-api.alphacode.com.br";
    const MAX_PARALLEL_ORDERS = 120;
    
    private $connection;
    private $webhookService;
    private $util;

    public function __construct(\Doctrine\DBAL\Connection $connection,
                                \App\Service\WebhookService $webhookService,
                                \App\Util\Util $util) {

        $this->connection       = $connection;
        $this->webhookService   = $webhookService;
        $this->util             = $util;
    }

    public function welcome(Request $request, Response $response) {
        return $response->withJson([
            'success' => true,
            'message' => "It works",
            'data'    => []
        ]);
    }

    public function getCurrentAuthToken(Request $request, Response $response) {
        try {
            $authToken = $this->webhookService->getAuthToken();
    
            return $response->withJson([
                'authToken' => $authToken['token']
            ]);
        } catch(\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function getMerchantStatus(Request $request, Response $response) {
        $req = $request->getParsedBody();

        $merchantId = $req['merchantId'];

        $this->authToken = $this->webhookService->getAuthToken();

        if(empty($this->authToken)) {
            $response->withStatus(200);
            return 0;
        }

        $merchantStatus = json_decode($this->webhookService->getMerchantStatus($merchantId, $this->authToken['token']), true);

        return $response->withJson([
            'merchantStatus' => $merchantStatus
        ]);
    }
    
    public function getMerchantInterruptions(Request $request, Response $response) {
        $req = $request->getParsedBody();

        $merchantId = $req['merchantId'];

        $this->authToken = $this->webhookService->getAuthToken();

        if(empty($this->authToken)) {
            $response->withStatus(200);
            return 0;
        }

        $merchantInterruptions = json_decode($this->webhookService->getMerchantInterruptions($merchantId, $this->authToken['token']), true);

        return $response->withJson([
            'merchantInterruption' => $merchantInterruptions
        ]);
    }
    
    public function requestCancelOrder(Request $request, Response $response) {
        $req = $request->getParsedBody();

        $orderId = $req['orderId'];

        $this->authToken = $this->webhookService->getAuthToken();

        if(empty($this->authToken)) {
            $response->withStatus(200);
            return 0;
        }

        $cancelStatus = $this->webhookService->requestCancelOrder($orderId, $this->authToken['token']);
        
        if($cancelStatus !== 202) {
            $error = true;
            $message = 'Não foi possível solicitar o cancelamento do pedido';
        } else {
            $error = false;
            $message = 'Solicitação de cancelamento realizada com sucesso para a plataforma do iFood';
        }

        return $response->withJson([
            'Error' => $error,
            'Message' => $message
        ]);
    }
    
    public function runPollingService(Request $request, Response $response) {
        try {
            $req = $request->getParsedBody();
            
            $this->authToken = $this->webhookService->getAuthToken();
            
            if(empty($this->authToken)) {
                $response->withStatus(200);
                return 0;
            }
            
            $unfilteredEvents = json_decode($this->webhookService->getEvents($this->authToken['token']), true);
            
            if(!empty($unfilteredEvents)){
                $this->webhookService->saveEvents($unfilteredEvents);   
            }
            
            return $response->withJson([
                'success' => true,
                'message' => 'pooling realizado com sucesso',
                'result'  => []
            ]);
        } catch(\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }
    
    public function runAckService(Request $request, Response $response) {
        try {
            $req = $request->getParsedBody();
            
            $this->authToken = $this->webhookService->getAuthToken();
            
            if(empty($this->authToken)) {
                $response->withStatus(200);
                return 0;
            }
            
            $this->webhookService->runPendingACKs($this->authToken['token']);   
            
            return $response->withJson([
                'success' => true,
                'message' => 'ack realizado com sucesso',
                'result'  => []
            ]);
        } catch(\Exception $e){
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function processParallelOrders(Request $request, Response $response) {
        $req = $request->getParsedBody();
            
        $this->authToken = $this->webhookService->getAuthToken();
        
        if(empty($this->authToken)) {
            $response->withStatus(200);
            return 0;
        }

        $processingErrors = array();

        $responseOrders = [];
        $running = null;
        $handles = [];
        $c = 0;

        $numberOfPendingOrders = 0;

        try {
            $orders = $this->util->readEventFiles("order");
            
            $numberOfPendingOrders = count($orders);

            if($numberOfPendingOrders > 0) {
                
                //IFOOD URL
                $url = self::MERCHANT_API_BASEURL . '/order/v1.0/orders/';
                
                //TEKNISA SANDBOX API URL
                //$url = 'http://192.168.120.98:9091/odhen-ifood-bench/src/public/index.php/order/v1.0/orders/';
                $header = array(
                    "Content-Type: application/json",
                    "Authorization: bearer ".$this->authToken['token']
                );

                foreach($orders as $order) {
                    $handles[$c] = curl_init();

                    curl_setopt_array($handles[$c], array(
                        CURLOPT_URL            => $url.$order['orderId'],
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_ENCODING       => '',
                        CURLOPT_MAXREDIRS      => 10,
                        CURLOPT_TIMEOUT        => 45,
                        CURLOPT_CONNECTTIMEOUT => 5,
                        CURLOPT_FOLLOWLOCATION => true,
						CURLOPT_SSL_VERIFYPEER => false,
                        CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
                        CURLOPT_CUSTOMREQUEST  => 'GET',
                        CURLOPT_HTTPHEADER     => $header
                    ));

                    $c++;

                    if($c >= self::MAX_PARALLEL_ORDERS) {
                        break;
                    }
                }

                $mh = curl_multi_init();
                for($i = 0; $i < $c; $i++) {
                    curl_multi_add_handle($mh, $handles[$i]);
                }
                    
                do {
                    curl_multi_exec($mh, $running);
                } while ($running > 0);

                for($i = 0; $i < $c; $i++){
                    $info = curl_multi_getcontent($handles[$i]);
                    $orderData = json_decode($info, true);
                    if(!isset($orderData['error'])){
                        array_push($responseOrders, $orderData);   
                    }
                }

                for($i = 0; $i < $c; $i++) {
                    curl_multi_remove_handle($mh, $handles[$i]);
                }

                curl_multi_close($mh);


                $integratedOrders = [];
                $running = null;
                $handles = [];
                $c = 0;

                foreach($responseOrders as $responseOrder) {
                    $clientData = $this->webhookService->getClientData($responseOrder['merchant']['id']);
                    
                    $clientName = $clientData['NOME'];
                    $clientType = $clientData['TIPO'];
                    $dbName     = $clientData['DBNAME'];
                    
                    if($clientType == 'hosting' || $clientType == 'saas'){
                        $header = array(
                            "Content-type: application/json",
                            "x-client-name: $clientName",
                            "x-client-type: $clientType",
                            "x-client-env: $dbName"
                        );
                    } else{
                        $header = array(
                            "Content-Type: application/json"
                        );
                    }
                    
                    $handles[$c] = curl_init();

                    curl_setopt_array($handles[$c], array(
                        CURLOPT_URL            => $clientData['URL']."/start",
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_ENCODING       => '',
                        CURLOPT_MAXREDIRS      => 10,
                        CURLOPT_TIMEOUT        => 45,
                        CURLOPT_CONNECTTIMEOUT => 5,
                        CURLOPT_FOLLOWLOCATION => true,
						CURLOPT_SSL_VERIFYPEER => false,
                        CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
                        CURLOPT_CUSTOMREQUEST  => 'POST',
                        CURLOPT_POSTFIELDS     => json_encode([
                            "orderData" => $responseOrder,
                            "authToken" => $this->authToken['token']
                        ]),
                        CURLOPT_HTTPHEADER     => $header
                    ));

                    $c++;

                    if($c >= self::MAX_PARALLEL_ORDERS) {
                        break;
                    }
                }

                $mh = curl_multi_init();
                for($i = 0; $i < $c; $i++) {
                    curl_multi_add_handle($mh, $handles[$i]);
                }
                    
                do {
                    curl_multi_exec($mh, $running);
                } while ($running > 0);
                
                for($i = 0; $i < $c; $i++){
                    $info = curl_multi_getcontent($handles[$i]);
                    array_push($integratedOrders, json_decode($info, true));
                }

                for($i = 0; $i < $c; $i++) {
                    curl_multi_remove_handle($mh, $handles[$i]);
                }

                curl_multi_close($mh);

                //temporary fix for takeout orders
                foreach($responseOrders as $responseOrder) {
                    $fileName = $orders[$responseOrder['id']]['id'] . ".json";
                    $this->util->manipulateEventFile("processed", $fileName, json_encode($orders[$responseOrder['id']]));
                }
            
                return $response->withJson([
                    'success' => true,
                    'message' => 'orders realizado com sucesso',
                    'result'  => $integratedOrders
                ]);

            }
            
            return $response->withJson([
                'success' => true,
                'message' => 'orders realizado com sucesso',
                'result'  => []
            ]);
        } catch(\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function processParallelCancellationRequests(Request $request, Response $response) {
        $req = $request->getParsedBody();
            
        $this->authToken = $this->webhookService->getAuthToken();
        
        if(empty($this->authToken)) {
            $response->withStatus(200);
            return 0;
        }

        $processingErrors = array();

        $responseOrders = [];
        $running = null;
        $handles = [];
        $c = 0;

        $numberOfPendingCancelRequests = 0;

        try {
            $cancelRequests = $this->util->readEventFiles("cancelRequest");
            
            $numberOfPendingCancelRequests = count($cancelRequests);

            if($numberOfPendingCancelRequests > 0) {
                
                //IFOOD URL
                $url = self::MERCHANT_API_BASEURL . '/order/v1.0/orders/';
                
                //TEKNISA SANDBOX API URL
                //$url = 'http://192.168.120.98:9091/odhen-ifood-bench/src/public/index.php/order/v1.0/orders/';
                $header = array(
                    "Content-Type: application/json",
                    "Authorization: bearer ".$this->authToken['token']
                );

                foreach($cancelRequests as $cancelRequest) {
                    $handles[$c] = curl_init();

                    curl_setopt_array($handles[$c], array(
                        CURLOPT_URL            => $url.$cancelRequest['orderId'],
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_ENCODING       => '',
                        CURLOPT_MAXREDIRS      => 10,
                        CURLOPT_TIMEOUT        => 45,
                        CURLOPT_CONNECTTIMEOUT => 5,
                        CURLOPT_FOLLOWLOCATION => true,
						CURLOPT_SSL_VERIFYPEER => false,
                        CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
                        CURLOPT_CUSTOMREQUEST  => 'GET',
                        CURLOPT_HTTPHEADER     => $header
                    ));

                    $c++;

                    if($c >= self::MAX_PARALLEL_ORDERS) {
                        break;
                    }
                }

                $mh = curl_multi_init();
                for($i = 0; $i < $c; $i++) {
                    curl_multi_add_handle($mh, $handles[$i]);
                }
                    
                do {
                    curl_multi_exec($mh, $running);
                } while ($running > 0);

                for($i = 0; $i < $c; $i++){
                    $info = curl_multi_getcontent($handles[$i]);
                    array_push($responseOrders, json_decode($info, true));
                }

                for($i = 0; $i < $c; $i++) {
                    curl_multi_remove_handle($mh, $handles[$i]);
                }

                curl_multi_close($mh);

                $requestCancelOrders = [];
                $running = null;
                $handles = [];
                $c = 0;

                foreach($responseOrders as $responseOrder) {
                    $clientData = $this->webhookService->getClientData($responseOrder['merchant']['id']);
                    
                    $clientName = $clientData['NOME'];
                    $clientType = $clientData['TIPO'];
                    $dbName     = $clientData['DBNAME'];
                    
                    if($clientType == 'hosting' || $clientType == 'saas'){
                        $header = array(
                            "Content-type: application/json",
                            "x-client-name: $clientName",
                            "x-client-type: $clientType",
                            "x-client-env: $dbName"
                        );
                    } else {
                        $header = array(
                            "Content-Type: application/json"
                        );
                    }
                    
                    $handles[$c] = curl_init();

                    curl_setopt_array($handles[$c], array(
                        CURLOPT_URL            => $clientData['URL']."/cancelOrderRequest",
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_ENCODING       => '',
                        CURLOPT_MAXREDIRS      => 10,
                        CURLOPT_TIMEOUT        => 45,
                        CURLOPT_CONNECTTIMEOUT => 5,
                        CURLOPT_FOLLOWLOCATION => true,
						CURLOPT_SSL_VERIFYPEER => false,
                        CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
                        CURLOPT_CUSTOMREQUEST  => 'POST',
                        CURLOPT_POSTFIELDS     => json_encode([
                            "orderData" => $responseOrder,
                            "authToken" => $this->authToken['token']
                        ]),
                        CURLOPT_HTTPHEADER     => $header
                    ));

                    $c++;

                    if($c >= self::MAX_PARALLEL_ORDERS) {
                        break;
                    }
                }

                $mh = curl_multi_init();
                for($i = 0; $i < $c; $i++) {
                    curl_multi_add_handle($mh, $handles[$i]);
                }
                    
                do {
                    curl_multi_exec($mh, $running);
                } while ($running > 0);
                
                for($i = 0; $i < $c; $i++){
                    $info = curl_multi_getcontent($handles[$i]);
                    array_push($requestCancelOrders, json_decode($info, true));
                }

                for($i = 0; $i < $c; $i++) {
                    curl_multi_remove_handle($mh, $handles[$i]);
                }
                
                foreach($requestCancelOrders as $requestCancelOrder) {
                    if($requestCancelOrder['success'] === true) {
                        $fileName = $cancelRequests[$requestCancelOrder['orderId']]['id'] . ".json";
                        $this->util->manipulateEventFile("processed", $fileName, json_encode($cancelRequests[$requestCancelOrder['orderId']]));
                    }
                }

                curl_multi_close($mh);

                return $response->withJson([
                    'success' => true,
                    'message' => 'cancel requests realizado com sucesso',
                    'result'  => []
                ]);
            }
        } catch(\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function processParallelCancellations(Request $request, Response $response) {

        $req = $request->getParsedBody();
            
        $this->authToken = $this->webhookService->getAuthToken();
        
        if(empty($this->authToken)) {
            $response->withStatus(200);
            return 0;
        }

        $processingErrors = array();

        $responseOrders = [];
        $running = null;
        $handles = [];
        $c = 0;

        $numberOfPendingCancellations = 0;

        try {
            $cancellations = $this->util->readEventFiles("cancelled");
            
            $numberOfPendingCancellations = count($cancellations);

            if($numberOfPendingCancellations > 0) {
                
                //IFOOD URL
                $url = self::MERCHANT_API_BASEURL . '/order/v1.0/orders/';
                
                //TEKNISA SANDBOX API URL
                //$url = 'http://192.168.120.98:9091/odhen-ifood-bench/src/public/index.php/order/v1.0/orders/';
                $header = array(
                    "Content-Type: application/json",
                    "Authorization: bearer " . $this->authToken['token']
                );

                foreach($cancellations as $cancellation) {
                    $handles[$c] = curl_init();

                    curl_setopt_array($handles[$c], array(
                        CURLOPT_URL            => $url.$cancellation['orderId'],
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_ENCODING       => '',
                        CURLOPT_MAXREDIRS      => 10,
                        CURLOPT_TIMEOUT        => 45,
                        CURLOPT_CONNECTTIMEOUT => 5,
                        CURLOPT_FOLLOWLOCATION => true,
						CURLOPT_SSL_VERIFYPEER => false,
                        CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
                        CURLOPT_CUSTOMREQUEST  => 'GET',
                        CURLOPT_HTTPHEADER     => $header
                    ));

                    $c++;

                    if($c >= self::MAX_PARALLEL_ORDERS) {
                        break;
                    }
                }

                $mh = curl_multi_init();
                for($i = 0; $i < $c; $i++) {
                    curl_multi_add_handle($mh, $handles[$i]);
                }
                    
                do {
                    curl_multi_exec($mh, $running);
                } while ($running > 0);

                for($i = 0; $i < $c; $i++){
                    $info = curl_multi_getcontent($handles[$i]);
                    array_push($responseOrders, json_decode($info, true));
                }

                for($i = 0; $i < $c; $i++) {
                    curl_multi_remove_handle($mh, $handles[$i]);
                }

                curl_multi_close($mh);

                $cancelledOrders = [];
                $running = null;
                $handles = [];
                $c = 0;

                foreach($responseOrders as $responseOrder) {
                    $clientData = $this->webhookService->getClientData($responseOrder['merchant']['id']);
                    
                    $clientName = $clientData['NOME'];
                    $clientType = $clientData['TIPO'];
                    $dbName     = $clientData['DBNAME'];
                    
                    if($clientType == 'hosting' || $clientType == 'saas'){
                        $header = array(
                            "Content-type: application/json",
                            "x-client-name: $clientName",
                            "x-client-type: $clientType",
                            "x-client-env: $dbName"
                        );
                    } else{
                        $header = array(
                            "Content-Type: application/json"
                        );
                    }
                    
                    $handles[$c] = curl_init();

                    curl_setopt_array($handles[$c], array(
                        CURLOPT_URL            => $clientData['URL']."/cancelOrder",
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_ENCODING       => '',
                        CURLOPT_MAXREDIRS      => 10,
                        CURLOPT_TIMEOUT        => 45,
                        CURLOPT_CONNECTTIMEOUT => 5,
                        CURLOPT_FOLLOWLOCATION => true,
						CURLOPT_SSL_VERIFYPEER => false,
                        CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
                        CURLOPT_CUSTOMREQUEST  => 'POST',
                        CURLOPT_POSTFIELDS     => json_encode([
                            "orderData" => $responseOrder,
                            "authToken" => $this->authToken['token']
                        ]),
                        CURLOPT_HTTPHEADER     => $header
                    ));

                    $c++;

                    if($c >= self::MAX_PARALLEL_ORDERS) {
                        break;
                    }
                }

                $mh = curl_multi_init();
                for($i = 0; $i < $c; $i++) {
                    curl_multi_add_handle($mh, $handles[$i]);
                }
                    
                do {
                    curl_multi_exec($mh, $running);
                } while ($running > 0);
                
                for($i = 0; $i < $c; $i++){
                    $info = curl_multi_getcontent($handles[$i]);
                    array_push($cancelledOrders, json_decode($info, true));
                }

                for($i = 0; $i < $c; $i++) {
                    curl_multi_remove_handle($mh, $handles[$i]);
                }
                
                foreach($cancelledOrders as $cancelledOrder) {
                    if($cancelledOrder['success'] === true) {
                        $fileName = $cancellations[$cancelledOrder['orderId']]['id'] . ".json";
                        $this->util->manipulateEventFile("processed", $fileName, json_encode($cancellations[$cancelledOrder['orderId']]));
                    }
                }

                curl_multi_close($mh);
                
                return $response->withJson([
                    'success' => true,
                    'message' => 'cancellation realizado com sucesso',
                    'result'  => []
                ]);
            }
        } catch(\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function processParallelDispatch(Request $request, Response $response) {

        $req = $request->getParsedBody();
            
        $this->authToken = $this->webhookService->getAuthToken();
        
        if(empty($this->authToken)) {
            $response->withStatus(200);
            return 0;
        }

        $processingErrors = array();

        $running = null;
        $handles = [];
        $c = 0;

        $numberOfMerchants = 0;

        try {
            $merchants = $this->webhookService->getMerchantsForDispatch();
            
            $numberOfMerchants = count($merchants);

            if($numberOfMerchants > 0) {
                
                foreach($merchants as $merchant) {
                    
                    $clientName = $merchant['NOME'];
                    $clientType = $merchant['TIPO'];
                    $dbName     = $merchant['DBNAME'];
                    
                    if($clientType == 'hosting' || $clientType == 'saas') {
                        $header = array(
                            "Content-type: application/json",
                            "x-client-name: $clientName",
                            "x-client-type: $clientType",
                            "x-client-env: $dbName"
                        );
                    } else{
                        $header = array(
                            "Content-Type: application/json"
                        );
                    }
                    
                    $handles[$c] = curl_init();

                    curl_setopt_array($handles[$c], array(
                        CURLOPT_URL            => $merchant['URL']."/dispatchOrders",
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_ENCODING       => '',
                        CURLOPT_MAXREDIRS      => 10,
                        CURLOPT_TIMEOUT        => 45,
                        CURLOPT_CONNECTTIMEOUT => 5,
                        CURLOPT_FOLLOWLOCATION => true,
						CURLOPT_SSL_VERIFYPEER => false,
                        CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
                        CURLOPT_CUSTOMREQUEST  => 'POST',
                        CURLOPT_POSTFIELDS     => json_encode([
                            "authToken" => $this->authToken['token']
                        ]),
                        CURLOPT_HTTPHEADER     => $header
                    ));

                    $c++;

                    if($c >= self::MAX_PARALLEL_ORDERS) {
                        break;
                    }
                }

                $mh = curl_multi_init();
                for($i = 0; $i < $c; $i++) {
                    curl_multi_add_handle($mh, $handles[$i]);
                }
                    
                do {
                    curl_multi_exec($mh, $running);
                } while ($running > 0);

                for($i = 0; $i < $c; $i++) {
                    curl_multi_remove_handle($mh, $handles[$i]);
                }

                curl_multi_close($mh);
                
                return $response->withJson([
                    'success' => true,
                    'message' => 'dispatch realizado com sucesso',
                    'result'  => []
                ]);
            }
        } catch(\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function getClientList(Request $request, Response $response) {
        try {
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->getClientList()
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }
    
    public function getWebhookParams(Request $request, Response $response) {
        try {
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->getWebhookParams()
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }
    
    public function updateParams(Request $request, Response $response){
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->updateParams($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function addClient(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->addClient($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function editClient(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->editClient($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function deleteClient(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "Cliente deletado com sucesso!",
                'data'    => $this->webhookService->deleteClient($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function addToken(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->addToken($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function editToken(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->editToken($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function deleteToken(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "Token deletado com sucesso!",
                'data'    => $this->webhookService->deleteToken($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }
    
    public function cleanToken(Request $request, Response $response) {
        try {
            return $response->withJson([
                'success' => true,
                'message' => "Token removido com sucesso!",
                'data'    => $this->webhookService->cleanToken()
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function renovateToken(Request $request, Response $response) {
        try {
            return $response->withJson([
                'success' => true,
                'message' => "Token renovado com sucesso!",
                'data'    => $this->webhookService->getAuthTokens()
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }
}